package djmil.cordacheckers.api;

import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.GameView;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.ReqGameBoardMove;
import djmil.cordacheckers.cordaclient.dao.flow.arguments.RspFailureException;
import djmil.cordacheckers.user.HoldingIdentityResolver;
import djmil.cordacheckers.user.User;

@RestController
@RequestMapping("api/game")
public class GameController {

    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @GetMapping
    public ResponseEntity<List<GameView>> list(
            @AuthenticationPrincipal User player) {
        try {
            final List<GameView> gsList = cordaClient.gameStateList(
                    player.getHoldingIdentity());

            return ResponseEntity.ok(gsList);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    @PutMapping("/{uuid}/surrender")
    public ResponseEntity<GameView> surrender(
            @AuthenticationPrincipal User issuer,
            @PathVariable UUID uuid) {
        try {
            final GameView finishedGame = cordaClient.gameBoardSurrender(
                    issuer.getHoldingIdentity(),
                    uuid);

            return ResponseEntity.ok(finishedGame);

        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    @PutMapping("/{uuid}/drawreq")
    public ResponseEntity<GameView> drawRequest(
            @AuthenticationPrincipal User issuer,
            @PathVariable UUID uuid) {
        try {
            final GameView drawReqGame = cordaClient.gameDrawRequest(
                    issuer.getHoldingIdentity(),
                    uuid);

            return ResponseEntity.ok(drawReqGame);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    @PutMapping("/{uuid}/drawacc")
    public ResponseEntity<GameView> drawAccept(
            @AuthenticationPrincipal User issuer,
            @PathVariable UUID uuid) {
        try {
            final GameView drawAccGame = cordaClient.gameDrawAccept(
                    issuer.getHoldingIdentity(),
                    uuid);

            return ResponseEntity.ok(drawAccGame);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    @PutMapping("/{uuid}/drawrej")
    public ResponseEntity<GameView> drawReject(
            @AuthenticationPrincipal User issuer,
            @PathVariable UUID uuid) {
        try {
            final GameView drawRejGame = cordaClient.gameDrawReject(
                    issuer.getHoldingIdentity(),
                    uuid);

            return ResponseEntity
                    .ok(drawRejGame);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }

    @PutMapping("/{uuid}/move")
    public ResponseEntity<GameView> move(
            @AuthenticationPrincipal User issuer,
            @PathVariable UUID uuid,
            @RequestBody ReqGameBoardMove request) {
        try {
            final GameView movedGame = cordaClient.gameBoardMove(
                    issuer.getHoldingIdentity(),
                    uuid,
                    request.move(),
                    request.message());

            return ResponseEntity
                    .ok(movedGame);
        } catch (RspFailureException e) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, e.getMessage());
        }
    }
}
