package djmil.cordacheckers.states;

import java.security.PublicKey;
import java.util.List;
import java.util.UUID;

import djmil.cordacheckers.checkers.Stone;
import net.corda.v5.base.annotations.CordaSerializable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.ledger.utxo.ContractState;

@CordaSerializable
public abstract class GameState implements ContractState {
    final MemberX500Name whitePlayer;
    final MemberX500Name blackPlayer;
    final UUID gameUuid;
    final String message;
    final List<PublicKey> participants;

    GameState(MemberX500Name whitePlayer, MemberX500Name blackPlayer, UUID gameUuid, 
            String message, List<PublicKey> participants) {
        this.whitePlayer = whitePlayer;
        this.blackPlayer = blackPlayer;
        this.gameUuid = gameUuid;
        this.message = message;
        this.participants = participants;
    }

    public MemberX500Name getWhitePlayer() {
        return whitePlayer;
    }

    public MemberX500Name getBlackPlayer() {
        return blackPlayer;
    }

    public UUID getGameUuid() {
        return gameUuid;
    }

    public String getMessage() {
        return message;
    }

    @Override
    public List<PublicKey> getParticipants() {
        return participants;
    }

    public MemberX500Name getOpponentName(MemberX500Name playerName) throws NotInvolved {
        if (playerName.compareTo(whitePlayer) == 0)
            return blackPlayer;
        if (playerName.compareTo(blackPlayer) == 0)
            return whitePlayer;
        throw new NotInvolved(playerName, this.getClass(), gameUuid);
    }

    public Stone.Color getPlayerColor(MemberX500Name playerName) throws NotInvolved {
        if (playerName.compareTo(whitePlayer) == 0)
            return Stone.Color.WHITE;
        if (playerName.compareTo(blackPlayer) == 0)
            return Stone.Color.BLACK;
        throw new NotInvolved(playerName, this.getClass(), gameUuid);
    }

    public static class NotInvolved extends RuntimeException {
        public <T> NotInvolved(MemberX500Name name, Class<T> clazz, UUID uuid) {
            super(name +" is not involved in " +clazz.getSimpleName() +" UUID " +uuid);
        }
    }

}
