package djmil.cordacheckers.gameproposal;

import java.time.Duration;
import java.time.Instant;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameCommand;
import djmil.cordacheckers.gamestate.CommitTrx;
import djmil.cordacheckers.gamestate.FlowResponce;
import djmil.cordacheckers.gamestate.GetFlow;
import djmil.cordacheckers.gamestate.View;
import djmil.cordacheckers.gamestate.ViewBuilder;
import djmil.cordacheckers.states.GameProposalState;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.common.NotaryLookup;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

public class RejectFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(RejectFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public MemberLookup memberLookup;

     @CordaInject
     public NotaryLookup notaryLookup;

     @CordaInject
     public UtxoLedgerService utxoLedgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          SecureHash utxoTrxId = null;

          try {
               final GameCommand command = new GameCommand(GameCommand.Action.GAME_PROPOSAL_REJECT);

               final UUID gameUuid = UUID.fromString(requestBody.getRequestBody());

               final StateAndRef<GameState> gameProposalSar = this.flowEngine
                    .subFlow(new GetFlow(gameUuid));
               final GameProposalState gameProposal = (GameProposalState)gameProposalSar.getState().getContractState();

               final UtxoSignedTransaction gameProposalRejectTrx = utxoLedgerService.createTransactionBuilder()
                    .addCommand(command)
                    .addInputState(gameProposalSar.getRef())
                    .addSignatories(gameProposal.getParticipants())
                    .setNotary(gameProposalSar.getState().getNotaryName())
                    .setTimeWindowUntil(Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
                    .toSignedTransaction();

               utxoTrxId = this.flowEngine
                    .subFlow(new CommitTrx(gameProposalRejectTrx, gameProposal.getParticipants()));

               final View gameStateView = this.flowEngine
                    .subFlow(new ViewBuilder(utxoTrxId));

               return new FlowResponce(gameStateView, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          } 
          catch (Exception e) {
               log.warn(requestBody + " [ERROR] " +e.toString());
               return new FlowResponce(e, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          }
     }

}
